--		BOX PUSHER
--			by HugoBDesigner

love.filesystem.enumerate = love.filesystem.enumerate or love.filesystem.getDirectoryItems
love.graphics.drawq = love.graphics.drawq or love.graphics.draw
fdraw = love.graphics.draw
love.graphics.setDefaultImageFilter = love.graphics.setDefaultImageFilter or love.graphics.setDefaultFilter
love.graphics.setMode = love.graphics.setMode or love.window.setMode
love.graphics.setLine = love.graphics.setLine or love.graphics.setLineWidth
love.filesystem.mkdir = love.filesystem.mkdir or love.filesystem.createDirectory

function love.load()
	love.graphics.setDefaultImageFilter("nearest", "nearest")
	scale = 2
	grid = {}
	rawfiles = love.filesystem.enumerate("maps")
	files = {}
	maps = {}
	font = love.graphics.newFont("font.ttf", 6*scale) --Century Gothic
	fontmed = love.graphics.newFont("font.ttf", 16*scale) --Century Gothic
	fontbig = love.graphics.newFont("font.ttf", 48*scale) --Century Gothic
	
	
	--------------------------------------------------------------------------------
	-- WOOO! Custom Intro I made a looong time ago being used for the first time! --
	--------------------------------------------------------------------------------
	introraw = love.filesystem.enumerate("Intro")
	introimages = {}
	intro = 1
	introdelay = 3
	local a = 0
	for i, v in pairs(introraw) do
		if string.sub(v, -4, -1) == ".png" then
			a = a+1
			introimages[a] = {}
			introimages[a].image = love.graphics.newImage("Intro/"..v)
			introimages[a].timer = .025*introdelay
		end
	end
	introimages[1].timer=.5*introdelay
	introimages[11].timer=.05*introdelay
	introimages[23].timer=.075*introdelay
	introimages[31].timer=.05*introdelay
	introimages[36].timer=.05*introdelay
	introimages[59].timer=.25*introdelay
	introimages[60].timer=.5*introdelay
	
	
	
	windowwidth = 400
	windowheight = 300
	for i = 1, #rawfiles do
		if string.sub(rawfiles[i], -4, -1) == ".png" then
			table.insert(files, rawfiles[i])
			table.insert(maps, love.image.newImageData("maps/" .. rawfiles[i]))
		end
	end
	currentmap = 1
	playerimg = love.graphics.newImage("graphics/player.png")
	playerquads = {
		["down1"] =	love.graphics.newQuad( 0,  0, 16, 16, 64, 32),
		["up1"] =	love.graphics.newQuad(16,  0, 16, 16, 64, 32),
		["left1"] =	love.graphics.newQuad(32,  0, 16, 16, 64, 32),
		["right1"] =love.graphics.newQuad(48,  0, 16, 16, 64, 32),
		
		["down2"] =	love.graphics.newQuad( 0, 16, 16, 16, 64, 32),
		["up2"] =	love.graphics.newQuad(16, 16, 16, 16, 64, 32),
		["left2"] =	love.graphics.newQuad(32, 16, 16, 16, 64, 32),
		["right2"] =love.graphics.newQuad(48, 16, 16, 16, 64, 32)
	}
	playerquad = "down1"
	boximg = love.graphics.newImage("graphics/box.png")
	boxonimg = love.graphics.newImage("graphics/boxon.png")
	dotimg = love.graphics.newImage("graphics/dot.png")
	wallimg = love.graphics.newImage("graphics/wall.png")
	backimg = love.graphics.newImage("graphics/back.png")
	fieldimg = love.graphics.newImage("graphics/field.png")
	congrats = love.graphics.newImage("graphics/congrats.png")
	state = "intro"
	maxlevel = 1
	menuselection = {}
	menuselection.x = 1
	menuselection.y = 1
	menuselection.width = math.min(#files, 5)
	menuselection.height = math.ceil(#files/5)
	menuselection.color={255, 255, 155, 255}
	menuoffset = -20+menuselection.height*8+(menuselection.height-1)*2
	
	player = {}
	box = {}
	player.x = 1
	player.y = 1
	
	nextleveltimer = 0
	nextlevelmtime = 2
	
	loadmap(currentmap)
	loaddata()
	setscreen()
	
	-- Recording stuff
	if not love.filesystem.exists("records") then
		love.filesystem.mkdir( "records" )
	end
	recordingdata = {}
	isRecording = false
	timerecorded = 0 --Just so users can see how long they've been recording
	framesrecorded = 0 --Just so users can see how long they've been recording
	recordingtimer = 0
	
	recordingmaxtimer = .025
end

function love.update(dt)
	if nextleveltimer > 0 then
		if nextleveltimer - dt > 0 then
			nextleveltimer = nextleveltimer - dt
		else
			nextleveltimer = 0
			nextlevel()
		end
	end
	
	if isRecording then
		if recordingtimer >= recordingmaxtimer then
			local a = love.graphics.newScreenshot()
			recordingdata[#recordingdata+1] = a
			framesrecorded = framesrecorded + dt
			timerecorded = round(framesrecorded)
			recordingtimer = 0
		else
			recordingtimer = recordingtimer + dt
		end
	end
	
	if intro then
		if introimages[intro].timer-dt > 0 then
			introimages[intro].timer = introimages[intro].timer-dt
		else
			if intro+1>#introimages then
				intro = false
				loadmap(currentmap)
				loaddata()
			else
				intro = intro+1
			end
		end
	end
end

function love.draw()
	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.setBackgroundColor(0, 0, 0, 255)
	love.graphics.setFont(font)
	if state == "intro" and intro then
		love.graphics.setBackgroundColor(255, 255, 255, 255)
		local sw = (windowwidth*scale)/introimages[intro].image:getWidth()
		local posy = (windowwidth*scale/2-introimages[intro].image:getHeight()*sw/2)/2
		love.graphics.draw(introimages[intro].image, 0, posy, 0, sw, sw)
	elseif state == "game" then
		love.graphics.setColor(0, 205, 255, 255)
		love.graphics.setFont(fontmed)
		love.graphics.print("Level " .. currentmap, 4*scale, 4*scale)
		love.graphics.setFont(font)
		love.graphics.setColor(255, 255, 255, 255)
		for x = 1, #grid do
			for y = 1, #grid[x] do
				local posx = windowwidth/2-(#grid/2)*16+(x-1)*16
				local posy = windowheight/2-(#grid[1]/2)*16+(y-1)*16
				if grid[x][y] == "wall" then
					love.graphics.draw(wallimg, posx*scale, posy*scale, 0, scale, scale)
				elseif grid[x][y] == "dot" then
					love.graphics.draw(backimg, posx*scale, posy*scale, 0, scale, scale)
					love.graphics.draw(dotimg, posx*scale, posy*scale, 0, scale, scale)
				elseif grid[x][y] == "field" then
					love.graphics.draw(backimg, posx*scale, posy*scale, 0, scale, scale)
					love.graphics.draw(fieldimg, posx*scale, posy*scale, 0, scale, scale)
				elseif grid[x][y] == "box" then
					love.graphics.draw(boximg, posx*scale, posy*scale, 0, scale, scale)
				elseif grid[x][y] == "boxon" then
					love.graphics.draw(boxonimg, posx*scale, posy*scale, 0, scale, scale)
				else
					love.graphics.draw(backimg, posx*scale, posy*scale, 0, scale, scale)
				end
			end
		end
		local posx = windowwidth/2-(#grid/2)*16+(player.x-1)*16
		local posy = windowheight/2-(#grid[1]/2)*16+(player.y-1)*16
		love.graphics.draw(playerimg, playerquads[playerquad], posx*scale, posy*scale, 0, scale, scale)
		
		if nextleveltimer > 0 then
			love.graphics.draw(congrats, 0, 0, 0, scale, scale)
		end
	else
		local posx = windowwidth/2-(menuselection.width*8+(menuselection.width-1)*2)/2
		local posy = windowheight/2-(menuselection.height*8+(menuselection.height-1)*2)/2
		for x = 1, menuselection.width do
			for y = 1, menuselection.height do
				local selection = (y-1)*5+x
				if selection <= #files then
					if selection <= maxlevel then
						love.graphics.setColor(255, 255, 255, 155)
					else
						love.graphics.setColor(255, 255, 255, 55)
					end
					love.graphics.rectangle("fill", (posx-2+(x-1)*2+(x-1)*8)*scale, (posy-2+(y-1)*2+(y-1)*8)*scale+menuoffset*scale, 8*scale, 8*scale)
					love.graphics.setColor(255, 255, 255, 255)
					love.graphics.print(selection, (posx-2+(x-1)*2+(x-1)*8)*scale+4*scale-font:getWidth(selection)/2, (posy-2+(y-1)*2+(y-1)*8)*scale+menuoffset*scale)
				end
			end
		end
		love.graphics.setLine(2*scale)
		love.graphics.setColor(255, 255, 255, 255)
		local x, y = menuselection.x, menuselection.y
		love.graphics.setColor(unpack(menuselection.color))
		love.graphics.rectangle("line", (posx-2+(x-1)*2+(x-1)*8)*scale-scale, (posy-2+(y-1)*2+(y-1)*8)*scale+menuoffset*scale-scale, 10*scale, 10*scale)
		love.graphics.setLine(1)
		love.graphics.setColor(195, 255, 155, 255)
		love.graphics.setFont(fontbig)
		love.graphics.print("Box Pusher", windowwidth/2*scale-fontbig:getWidth("Box Pusher")/2, 20*scale)
	end
	if state ~= "intro" then
		love.graphics.setColor(255, 255, 255, 255)
		love.graphics.setFont(font)
		local otherstate = "game"
		if state == "game" then otherstate = "menu" end
		local othertext = "\"R\": Restart level"
		if state == "menu" then othertext = "" end
		love.graphics.print("\"M\": Return to " .. otherstate .. "    \"-\": Decrease scale    \"+\": Increase scale    " .. othertext, 4*scale, (windowheight-14)*scale)
	end
end

function love.keypressed(k, u)
	if state == "game" then
		if nextleveltimer <= 0 then
			if k=="w" or k=="up" then
				if player.y-1 > 0 then
					if grid[player.x][player.y-1] == "wall" then
						--Nothing really
					else
						if grid[player.x][player.y-1] == "box" or grid[player.x][player.y-1] == "boxon" then
							local newtile = "back"
							if grid[player.x][player.y-1] == "boxon" then newtile = "dot" end
							if player.y-2 > 0 and not (grid[player.x][player.y-2]=="box" or grid[player.x][player.y-2]=="boxon" or grid[player.x][player.y-2]=="wall" or grid[player.x][player.y-2]=="field") then
								setplayerquad("up")
								player.y = player.y-1
								grid[player.x][player.y] = newtile
								if grid[player.x][player.y-1] == "dot" then
									grid[player.x][player.y-1] = "boxon"
								else
									grid[player.x][player.y-1] = "box"
								end
							end
						else
							setplayerquad("up")
							player.y = player.y-1
						end
					end
				end
				checkdots()
			elseif k=="s" or k=="down" then
				if player.y+1 <= #grid[1] then
					if grid[player.x][player.y+1] == "wall" then
						--Nothing really
					else
						if grid[player.x][player.y+1] == "box" or grid[player.x][player.y+1] == "boxon" then
							local newtile = "back"
							if grid[player.x][player.y+1] == "boxon" then newtile = "dot" end
							if player.y+2 <= #grid[1] and not (grid[player.x][player.y+2]=="box" or grid[player.x][player.y+2]=="boxon" or grid[player.x][player.y+2]=="wall" or grid[player.x][player.y+2]=="field") then
								setplayerquad("down")
								player.y = player.y+1
								grid[player.x][player.y] = newtile
								if grid[player.x][player.y+1] == "dot" then
									grid[player.x][player.y+1] = "boxon"
								else
									grid[player.x][player.y+1] = "box"
								end
							end
						else
							setplayerquad("down")
							player.y = player.y+1
						end
					end
				end
				checkdots()
			elseif k=="a" or k=="left" then
				if player.x-1 > 0 then
					if grid[player.x-1][player.y] == "wall" then
						--Nothing really
					else
						if grid[player.x-1][player.y] == "box" or grid[player.x-1][player.y] == "boxon" then
							local newtile = "back"
							if grid[player.x-1][player.y] == "boxon" then newtile = "dot" end
							if player.x-2 > 0 and not (grid[player.x-2][player.y]=="box" or grid[player.x-2][player.y]=="boxon" or grid[player.x-2][player.y]=="wall" or grid[player.x-2][player.y]=="field") then
								setplayerquad("left")
								player.x = player.x-1
								grid[player.x][player.y] = newtile
								if grid[player.x-1][player.y] == "dot" then
									grid[player.x-1][player.y] = "boxon"
								else
									grid[player.x-1][player.y] = "box"
								end
							end
						else
							setplayerquad("left")
							player.x = player.x-1
						end
					end
				end
				checkdots()
			elseif k=="d" or k=="right" then
				if player.x+1 <= #grid then
					if grid[player.x+1][player.y] == "wall" then
						--Nothing really
					else
						if grid[player.x+1][player.y] == "box" or grid[player.x+1][player.y] == "boxon" then
							local newtile = "back"
							if grid[player.x+1][player.y] == "boxon" then newtile = "dot" end
							if player.x+2 <= #grid and not (grid[player.x+2][player.y]=="box" or grid[player.x+2][player.y]=="boxon" or grid[player.x+2][player.y]=="wall" or grid[player.x+2][player.y]=="field") then
								setplayerquad("right")
								player.x = player.x+1
								grid[player.x][player.y] = newtile
								if grid[player.x+1][player.y] == "dot" then
									grid[player.x+1][player.y] = "boxon"
								else
									grid[player.x+1][player.y] = "box"
								end
							end
						else
							setplayerquad("right")
							player.x = player.x+1
						end
					end
				end
				checkdots()
			end
		end
		
		if k=="r" then
			loadmap(currentmap)
			playerquad = "down1"
		elseif k=="m" then
			savedata()
			state = "menu"
			menuselection.x = math.mod(currentmap, menuselection.width)
			if menuselection.x == 0 then menuselection.x = menuselection.width end
			menuselection.y = math.ceil(currentmap/menuselection.width)
		end
	elseif state == "menu" then
		if k=="d" or k=="right" then
			if menuselection.x < menuselection.width then
				if menuselection.y == menuselection.height then
					if menuselection.x < math.mod(#files, menuselection.width) then
						menuselection.x = menuselection.x+1
					end
				else
					menuselection.x = menuselection.x+1
				end
			end
		elseif k=="a" or k=="left" then
			if menuselection.x > 1 then
				menuselection.x = menuselection.x-1
			end
		elseif k=="s" or k=="down" then
			if menuselection.y < menuselection.height then
				if menuselection.y == menuselection.height-1 then
					if menuselection.x <= math.mod(#files, menuselection.width) then
						menuselection.y = menuselection.y+1
					end
				else
					menuselection.y = menuselection.y+1
				end
			end
		elseif k=="w" or k=="up" then
			if menuselection.y > 1 then
				menuselection.y = menuselection.y-1
			end
		elseif k=="enter" or k=="return" or k=="kpenter" or k==" " then
			local selection = (menuselection.y-1)*5+menuselection.x
			print("selection = " .. selection)
			if selection > maxlevel then
				menuselection.color={255, 155, 155, 255}
			else
				loadmap(selection)
				state = "game"
			end
		elseif k=="m" then
			state = "game"
			loaddata()
		end
		
		print("menuselection.x = " .. menuselection.x)
		print("menuselection.y = " .. menuselection.y)
		print("menuselection.width = " .. menuselection.width)
		print("menuselection.height = " .. menuselection.height)
	end
	
	if k=="escape" then
		love.event.quit()
	elseif k=="-" and not intro then
		if scale > 1 then scale = scale - 1 end
		setscreen()
	elseif k=="=" and not intro then
		if scale < 4 then scale = scale + 1 end
		setscreen()
	end
	
	if not intro then
		if k == "f3" then
			
			-- PART 1: Get the screenshot itself and make it as an image
			
			local number = 1
			local lnumber = "0001"

			if not love.filesystem.exists("screenshots") then --make one
				love.filesystem.mkdir("screenshots")
			end

			local screenshot = love.graphics.newScreenshot()
			while love.filesystem.exists("screenshots/screenshot_" .. lnumber .. ".png") do
				number = number + 1
				if number >= 1 and number < 10 then
					lnumber = "000" .. tostring(number)
				elseif number >= 10 and number < 100 then
					lnumber = "00" .. tostring(number)
				elseif number >= 100 and number < 1000 then
					lnumber = "0" .. tostring(number)
				else
					lnumber = tostring(number)
				end
			end
			screenshot:encode("screenshots/screenshot_" .. lnumber .. ".png")
			
			
			--PART 2: Remove transparecy from screenshots. I noticed that they make transparent pixels
			
			local image = love.image.newImageData("screenshots/screenshot_" .. lnumber .. ".png")
			local imagewidth = image:getWidth()
			local imageheight = image:getHeight()
			for y = 0, imageheight-1 do
				for x = 0, imagewidth-1 do
					local r, g, b, a = image:getPixel( x, y )
					if a < 255 then
						a = 255
					end
					image:setPixel( x, y, r, g, b, a)
				end
			end
			image:encode("screenshots/screenshot_" .. lnumber .. ".png")
		end
		
		if k == "f9" then
			if isRecording then
				stoprecording()
			else
				startrecording()
			end
		end
		
		if k == "f10" then
			if isRecording then
				stoprecording(true)
			end
		end
		
		if k == "f11" then
			if isRecording then
				isRecording = false
				recordingdata = {}
				framesrecorded = 0
				timerecorded = 0
			end
		end
	end
end

function love.keyreleased(k, u)
	if state == "menu" then
		if k=="enter" or k=="return" or k=="kpenter" or k==" " then
			menuselection.color={255, 255, 155, 255}
		end
	end
end

function nextlevel()
	if currentmap+1 <= #files then
		currentmap = currentmap + 1
		loadmap(currentmap)
		if currentmap > maxlevel then
			maxlevel = currentmap
		end
	else
		state = "menu"
		menuselection.x = math.mod(currentmap, menuselection.width)
		if menuselection.x == 0 then menuselection.x = menuselection.width end
		menuselection.y = math.ceil(currentmap/menuselection.width)
		maxlevel = currentmap+1
	end
end

function love.mousepressed(x, y, b)
	
end

function setplayerquad(dir)
	local pdir = string.sub(playerquad, 1, -2)
	local pnum = string.sub(playerquad, -1, -1)
	local ndir = dir
	local nnum = "1"
	if pdir == ndir and pnum == "1" then nnum = "2" end
	playerquad = dir..nnum
end

function loadmap(map)
	local nmap = maps[map] or false
	
	if nmap then
		grid = {}
		for x = 1, nmap:getWidth() do
			grid[x] = {}
			for y = 1, nmap:getHeight() do
				local r, g, b, a = nmap:getPixel(x-1, y-1)
				if r<50 and g<50 and b<50 and a>200 then
					grid[x][y] = "wall"
				elseif r>200 and g>200 and b<50 and a>200 then
					grid[x][y] = "dot"
				elseif r<50 and g>200 and b<50 and a>200 then
					grid[x][y] = "field"
				elseif r<50 and g<50 and b>200 and a>200 then
					player.x = x
					player.y = y
					grid[x][y] = "back"
				elseif r>200 and g<50 and b<50 and a>200 then
					grid[x][y] = "box"
				elseif r>200 and g<50 and b>200 and a>200 then
					grid[x][y] = "boxon"
				else
					grid[x][y] = "back"
				end
			end
		end
		currentmap = map
	end
end

function love.graphics.draw(...)
	local args = { ... }
	if type(args[2]) == "userdata" then
		love.graphics.drawq(unpack(args))
	else
		fdraw(unpack(args))
	end
end

function setscreen()
	love.graphics.setMode(windowwidth*scale, windowheight*scale)
	font = love.graphics.newFont("font.ttf", 6*scale) --Century Gothic
	fontmed = love.graphics.newFont("font.ttf", 16*scale) --Century Gothic
	fontbig = love.graphics.newFont("font.ttf", 48*scale) --Century Gothic
end

function checkdots()
	local hasbox = false
	for x = 1, #grid do
		for y = 1, #grid[x] do
			if grid[x][y] == "box" then
				hasbox = true
				break
			end
		end
	end
	if hasbox == false then
		nextleveltimer = nextlevelmtime
	end
end

function savedata()
	local txt = "state = "..state.."\r\nmaxlevel = "..maxlevel.."\r\nlastlevel = "..currentmap.."\r\n".."scale = "..scale
	if state == "game" then
		txt = txt.."\r\n".."map = "..#grid.."/"..#grid[1]
		for x = 1, #grid do
			for y = 1, #grid[x] do
				txt = txt .. "/" .. grid[x][y]
			end
		end
		txt = txt .. "\r\n" .. "player = " .. player.x .. "/" .. player.y .. "/" .. playerquad
	end
	love.filesystem.write("save.data", txt)
end

function loaddata()
	state = "menu"
	currentmap = 1
	loadmap(currentmap)
	if love.filesystem.exists("save.data") then
		local txt = love.filesystem.read("save.data")
		local data = txt:split("\r\n")
		for i = 1, #data do
			local data2 = data[i]:split(" = ")
			if data2[1] == "state" then
				state = data2[2]
			elseif data2[1] == "maxlevel" then
				maxlevel = tonumber(data2[2])
			elseif data2[1] == "lastlevel" then
				currentmap = tonumber(data2[2])
				menuselection.x = math.mod(currentmap, menuselection.width)
				if menuselection.x == 0 then menuselection.x = menuselection.width end
				menuselection.y = math.ceil(currentmap/menuselection.width)
			elseif data2[1] == "scale" then
				scale = tonumber(data2[2])
			elseif data2[1] == "player" then
				local data3 = data2[2]:split("/")
				player.x = tonumber(data3[1])
				player.y = tonumber(data3[2])
				playerquad = data3[3]
			elseif data2[1] == "map" then
				local data3 = data2[2]:split("/")
				local w = tonumber(data3[1])
				local h = tonumber(data3[2])
				grid = {}
				for x = 1, w do
					grid[x] = {}
					for y = 1, h do
						grid[x][y] = data3[((x-1)*h+y)+2] --I have no idea of why I had to invert it :S
					end
				end
			end
		end
	end
end

function string:split(delimiter) --Not by me
	local result = {}
	local from	= 1
	local delim_from, delim_to = string.find( self, delimiter, from	 )
	while delim_from do
		table.insert( result, string.sub( self, from , delim_from-1 ) )
		from = delim_to + 1
		delim_from, delim_to = string.find( self, delimiter, from  )
	end
	table.insert( result, string.sub( self, from  ) )
	return result
end

function startrecording() --By me!
	recordingdata = {}
	isRecording = true
end

function stoprecording(transparency) -- Got this from my Mari0 +Portal mod, but it's mine :)
	local transparency = transparency or false
	isRecording = false
	recordingtimer = 0
	if not love.filesystem.exists( "records" ) then
		love.filesystem.mkdir( "records" )
	end
	local a = "0001"
	local n = 1
	if love.filesystem.exists("records/record_" .. a) then
		while love.filesystem.exists("records/record_" .. a) do
			local zeros = ""
			n = n + 1
			if n >= 1 and n < 10 then
				zeros = "000"
			elseif n >= 10 and n < 100 then
				zeros = "00"
			elseif n >= 100 and n < 1000 then
				zeros = "0"
			end
			a = zeros .. tostring(n)
		end
	end
	n = 0
	love.filesystem.mkdir("records/record_" .. a)
	
	------------------
	
	for i = 1, #recordingdata do
		print("Recording saving at " .. tostring(round(100/#recordingdata*i, 2)) .. "%")
		local b = "0001"
		if love.filesystem.exists("records/record_" .. a .. "/" .. b .. ".png") then
			while love.filesystem.exists("records/record_" .. a .. "/" .. b .. ".png") do
				local zeros = ""
				n = n + 1
				if n >= 1 and n < 10 then
					zeros = "000"
				elseif n >= 10 and n < 100 then
					zeros = "00"
				elseif n >= 100 and n < 1000 then
					zeros = "0"
				end
				b = zeros .. tostring(n)
			end
		end
		
		if transparency then
			for x = 0, recordingdata[i]:getWidth()-1 do
				for y = 0, recordingdata[i]:getHeight()-1 do
					local r, g, b, a = recordingdata[i]:getPixel(x, y)
					a = 255
					recordingdata[i]:setPixel(x, y, r, g, b, a)
				end
			end
		end
		recordingdata[i]:encode("records/record_" .. a .. "/" .. b .. ".png")
	end
	
	recordingdata = {}
	
	framesrecorded = 0
	timerecorded = 0
end

function love.quit()
	savedata()
end